@@ -24,22 +24,28 @@ class Box(object):
     Box 打包解包
     """
 
-    # 继承时可修改
+    # 只能继承时修改
     header_attrs = HEADER_ATTRS
-    # 网络字节序. 继承时可修改
+    # 只能继承时修改. 网络字节序
     header_fmt_type = '!'
 
+    header_keys = None
     header_fmt = None
     header_len = None
 
     # 如果调用unpack成功的话，会置为True
-    _unpack_done = None
+    unpack_done = None
 
     body = None
 
     def __init__(self, init_data=None):
-        self.header_fmt = self.header_fmt_type + ''.join(value[0] for value in self.header_attrs.values())
-        self.header_len = struct.calcsize(self.header_fmt)
+        # 为了优化性能，不用每次都计算
+        # 验证过，继承后，这里的__class__是子类类型，所以可以放心继承
+        cls = self.__class__
+        if cls.header_keys is None or cls.header_fmt is None or cls.header_len is None:
+            cls.header_keys = cls.header_attrs.keys()
+            cls.header_fmt = cls.header_fmt_type + ''.join(value[0] for value in cls.header_attrs.values())
+            cls.header_len = struct.calcsize(cls.header_fmt)
 
         self.reset()
         if init_data and isinstance(init_data, dict):
@@ -64,12 +70,8 @@ class Box(object):
         # 什么也不需要做，因为不能让别人来赋值
         pass
 
-    @property
-    def unpack_done(self):
-        return self._unpack_done
-
     def reset(self):
-        self._unpack_done = False
+        self.unpack_done = False
 
         for k, v in self.header_attrs.items():
             setattr(self, k, v[1])
@@ -80,7 +82,7 @@ class Box(object):
         """
         打包
         """
-        values = [getattr(self, k) for k in self.header_attrs]
+        values = [getattr(self, k) for k in self.header_keys]
 
         return struct.pack(self.header_fmt, *values) + self.body
 
@@ -96,7 +98,10 @@ class Box(object):
             0: 继续收
         """
 
-        if len(buf) < self.header_len:
+        # len(buf) 改为 buf_len 之后，性能居然提升了不少
+        buf_len = len(buf)
+
+        if buf_len < self.header_len:
             # logger.error('buf.len(%s) should > header_len(%s)' % (len(buf), self.header_len))
             # raise ValueError('buf.len(%s) should > header_len(%s)' % (len(buf), HEADER_LEN))
             return 0
@@ -107,7 +112,7 @@ class Box(object):
             logger.error('unpack fail.', exc_info=True)
             return -1
 
-        dict_values = dict([(key, values[i]) for i, key in enumerate(self.header_attrs.keys())])
+        dict_values = dict([(key, values[i]) for i, key in enumerate(self.header_keys)])
 
         if hasattr(self, 'magic'):
             magic = dict_values.get('magic')
@@ -124,7 +129,7 @@ class Box(object):
             logger.error('there is no packet_len or body_len in header')
             return -3
 
-        if len(buf) < packet_len:
+        if buf_len < packet_len:
             # 还要继续收
             return 0
 
@@ -137,7 +142,7 @@ class Box(object):
 
         self.body = buf[self.header_len:packet_len]
 
-        self._unpack_done = True
+        self.unpack_done = True
 
         return self.packet_len
 
@@ -190,7 +195,7 @@ class Box(object):
         self.body = json.dumps(value)
 
     def __repr__(self):
-        values = [(k, getattr(self, k)) for k in self.header_attrs]
+        values = [(k, getattr(self, k)) for k in self.header_keys]
 
         values.append(('body', self.body))
 
